<?php
interface FileOperationCommand {
    public function execute();
    public function undo();
}

class RenameFile implements FileOperationCommand {
    protected $src;
    protected $target;

    public function __construct($src, $target) {
        $this->src = $src;
        $this->target = $target;
    }

    public function execute() {
        rename($this->src, $this->target);
    }

    public function undo() {
        rename($this->target, $this->src);
    }
}

class CopyFile implements FileOperationCommand {
    protected $src;
    protected $target;

    public function __construct($src, $target) {
        $this->src = $src;
        $this->target = $target;
    }

    public function execute() {
        copy($this->src, $this->target);
    }

    public function undo() {
        unlink($this->target);
    }
}

class Batch {
    protected $commands = array();

    public function add(FileOperationCommand $command) {
        $this->commands[] = $command;
    }

    public function execute() {
        for ($i = 0; $i < count($this->commands); $i++) {
            $this->commands[$i]->execute();
        }
    }

    public function undo() {
        for ($i = count($this->commands)-1; $i >=0 ; $i--) {
            $this->commands[$i]->undo();
        }
    }
}
?>