<?php
require_once 'Car.php';

class IOException extends Exception {}
class IndexOutOfBoundsException extends Exception {}

class CarList implements IteratorAggregate {

    protected $carDefinitions = array();
    protected $cars = array();


    public function __construct($csvFile) {
        if (!file_exists($csvFile)) {
            throw new IOException();
        }
        $fp = fopen($csvFile, 'r');
        while (false !== $line = fgetcsv($fp, 1024, ';')) {
            $this->carDefinitions[] = array(
                                        'manufacturer' => $line[0],
                                        'color'        => $line[1],
                                        'milage'       => $line[2],
                                      );
        }
    }

    public function countCars() {
        return count($this->carDefinitions);
    }

    public function getCar($pos) {
        if ($pos >= count($this->carDefinitions)) {
            throw new IndexOutOfBoundsException();
        }
        if (!isset($this->cars[$pos])) {
            $this->cars[$pos] = new Car($this->carDefinitions[$pos]['manufacturer'],
                                        $this->carDefinitions[$pos]['color'],
                                        $this->carDefinitions[$pos]['milage']);
        }
        return $this->cars[$pos];
    }

    public function getIterator() {
        return new CarListIterator($this);
    }
}

class CarListIterator implements Iterator {

    protected $pos = 0;
    protected $list;

    public function __construct(CarList $list) {
        $this->list = $list;
    }

    public function rewind() {
        $this->pos = 0;
    }

    public function key() {
        return $this->pos;
    }

    public function next() {
        $this->pos++;
    }

    public function valid() {
        if ($this->pos < $this->list->countCars()) {
            return true;
        }
        return false;
    }

    public function current() {
        return $this->list->getCar($this->pos);
    }
}