<?php
require_once 'Vehicle.php';
require_once 'Observable.php';

/**
 * Reprsentation eines beliebigen Autos,
 * das das Interface 'Vehicle' erfllt.
 */
class Car implements Vehicle, Observable {
    /**
     * Hersteller des Autos
     *
     * @var string
     */
    protected $manufacturer;

    /**
     * Farbe des Autos
     *
     * @var string
     */
    protected $color;

    /**
     * Kilometerstand des Autos
     *
     * @var integer
     */
    protected $milage;

    /**
     * Hchstgeschwindigkeit
     *
     * @var int
     */
    protected $maxSpeed;

    /**
     * Gibt an, ob der Motor schon gestartet wurde
     *
     * @var boolean
     */
    protected $engineStarted = false;

    /**
     * Speichert alle Observer
     *
     * @var array
     */
    protected $observers = array();

    /**
     * Konstruktor fr ein neues Auto
     *
     * @param string $manufacturer
     * @param string $color
     * @param integer $milage
     */
    public function __construct($manufacturer, $color, $milage = 0, $maxSpeed = 100) {
        $this->manufacturer = $manufacturer;
        $this->color = $color;
        $this->milage = $milage;
        $this->maxSpeed = $maxSpeed;
    }

    /**
     * Destruktor des Autos
     * Falls der Motor noch luft wird dieser ausgeschalten.
     */
    public function __destruct() {
        if ($this->engineStarted) {
        	$this->stopEngine();
        }
    }

    /**
     * Motor anlassen
     */
    public function startEngine() {
        $this->engineStarted = true;
    }

    /**
     * Einige Kilometer fahren
     *
     * @param integer $miles
     */
    public function moveForward($miles) {
        // Wenn der Motor nicht luft, kann nicht gefahren werden.
        if ($this->engineStarted !== true) {
        	return false;
        }
        // Kilometerstand erhhen
        $this->milage = $this->milage + $miles;
        $this->notify();
        return true;
    }

    /**
     * Motor wieder stoppen
     */
    public function stopEngine() {
        $this->engineStarted = false;
    }

    /**
     * Kilometerstand zurck geben
     *
     * @return integer
     */
    public function getMilage() {
        return $this->milage;
    }

    /**
     * Tagessatz ermitteln
     *
     * @param integer $days
     * @return float
     */
    public function getDailyRate($days = 1) {
        if ($days >= 7) {
        	return 65.90;
        }
        return 75.50;
    }

    /**
     * Hersteller zurckgeben
     *
     * @return string
     */
    public function getManufacturer() {
        return $this->manufacturer;
    }

    /**
     * Farbe zurckgeben
     *
     * @return string
     */
    public function getColor() {
        return $this->color;
    }

    /**
     * Gibt die Hchstgeschwindigkeit zurck
     *
     * @return integer
     */
    public function getMaxSpeed() {
        return $this->maxSpeed;
    }

    /**
     * Fgt neuen Observer hinzu
     *
     * @param Observer $observer
     */
    public function attach(Observer $observer) {
        $this->observers[] = $observer;
    }

    /**
     * Entfernt einen Observer
     *
     * @param Observer $observer
     */
    public function detach(Observer $observer) {
        $this->observers = array_diff($this->observers, array($observer));
    }

    /**
     * Informiert alle Observer ber eine Statusnderung.
     */
    public function notify() {
        foreach ($this->observers as $observer) {
        	$observer->update($this);
        }
    }
}
?>